/*
** Copyright (c) 2003 CA.  All rights reserved.  
** This software may not be duplicated, disclosed or reproduced in whole or 
** in part for any purpose except as authorized by the applicable license agreement, 
** without the express written authorization of CA. All authorized reproductions 
** must be marked with this language.
**
** TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS 
** SOFTWARE WITHOUT WARRANTY OF ANY KIND, INCLUDING 
** WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
** FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT 
** WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS 
** OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL, 
** INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS 
** INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY 
** ADVISED OF SUCH LOSS OR DAMAGE.
*/

// SiteMinder Java Authentication API sample.

package com.ca.siteminder.sdk.example.delay;

import java.util.Date;
import java.text.DateFormat;
import com.netegrity.policyserver.smapi.*;

import javax.security.cert.X509Certificate;
import javax.security.cert.CertificateException;
import java.util.regex.*;
/**
 * A simple SiteMinder Java Authentication Scheme.
 */

public class AuthApiDelaySample implements SmAuthScheme  {
    private static final int SCHEME_VERSION = SmAuthQueryResponse.SMAUTH_API_VERSION_V3;
    private static final String SCHEME_DESCRIPTION = "SiteMinder(tm) Example Java authentication scheme, (c) Netegrity 2003";


    /**
     * Returns information about the authentication scheme.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param request - The request code, SMAUTH_QUERY_DESCRIPTION or SMAUTH_QUERY_CREDENTIALS_REQ.
     * @param response - Holds methods by which query() returns the requested information.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus query(String parameter, String secret, SmAuthQueryCode request, SmAuthQueryResponse response) {

        log(" query : " + parameter + "," + secret + ", " + request + ", " + response);

        if (null == response) {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        if (SmAuthQueryCode.SMAUTH_QUERY_DESCRIPTION == request) {
            response.setResponseBuffer(SCHEME_DESCRIPTION);
            response.setResponseCode(SCHEME_VERSION);
        }
        else if (SmAuthQueryCode.SMAUTH_QUERY_CREDENTIALS_REQ == request) {
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_BASIC);
            response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_FORM_REQUIRED);
            response.setResponseBuffer(getRedirectURL(parameter));
            //response.setResponseBuffer(parameter);
            System.out.println(" query::Parameter : " + parameter);

        }
        else {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method so the authentication scheme can
     * perform its own initialization procedure. This method is invoked once
     * for each authentication scheme instance when it is first loaded.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */


    String getRedirectURL(String parameter) {
        String redirectURL = parameter;
        
        if (parameter.indexOf(';') != -1) {
            String[] params = parameter.split(";");
            redirectURL = params[0];
        }
        log(" redirectURL    : " + redirectURL );
        return redirectURL;
    }

    int getDelayTimeMilli(String parameter) {

        int delayTimeMilli = 0;
        
        if (parameter.indexOf(';') != -1) {
            String[] params = parameter.split(";");
            //redirectURL = params[0];
            
            try { 
                delayTimeMilli = Integer.parseInt(params[1]) * 1000;
            }
            catch (NumberFormatException e) {}
        }

        log(" delayTimeMilli : " + delayTimeMilli );

        return delayTimeMilli;
        
    }
    
    public SmAuthStatus  init(String parameter, String secret)  {


        // parameter format wants to be: 
        //      "http://<loginpage>/siteminderagent/forms/login.fcc;10" 
        // for 10 sec delay.

        log(" init::Parameter : " + parameter + " " + secret);

        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method during shutdown so the authentication
     * scheme can perform its own rundown procedure. This method is invoked
     * once for each authentication scheme instance during SiteMinder shutdown.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus release(String parameter, String secret) {
        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    void log(String msg) {
         System.out.println(getClass().getSimpleName() + " AuthApiDelay:: ['" + msg +"']");
    }

    void log(SmAuthenticationContext context, String msg) {
         context.getAPIContext().trace(getClass().getSimpleName(), "AuthApiDelay:: ['" + msg +"']");
//         System.out.println(this.getClass().getName() + " AuthApiDelay:: ['" + msg +"']");
         System.out.println(getClass().getSimpleName() + " AuthApiDelay:: ['" + msg +"']");
    }

    /**
     * SiteMinder invokes this method to authenticate user credentials.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param challengeReason - The reason for the original authentication challenge, or 0 if unknown.
     * @param context - Contains request context and methods to return message buffers.
     *
     * @return: an SmAuthenticationResult object.
     */


    public SmAuthenticationResult authenticate(String parameter, String secret, int challengeReason,  SmAuthenticationContext context) {
        // cannot do authentication without the authentication context

        log(context, " authenticate::Parameter : " + parameter + ", " + secret + ", " + challengeReason + ", " + context);


        if (null == context) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }

    	log(context, " " + context.getAPIContext().getVersion() +" ");

        UserCredentialsContext userCredentialsContext = context.getUserCredentialsContext();

        if ( userCredentialsContext == null) {
                log(context, "user Credentials are null");
        	return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        UserContext theUserContext = context.getUserContext();

        
        // If the scheme is designed not to disambiguate users, it should return SmAuthApi_NoUserContext.
        if ((null == theUserContext) || !theUserContext.isUserContext()) {
            return  new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }        

        // Reject the user if the password is not entered.
        UserCredentialsContext theUserCredentialsContext = context.getUserCredentialsContext();

        if (null == theUserCredentialsContext)  {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        // Check password is not empty 
        String thePassword = theUserCredentialsContext.getPassword();   
        if (thePassword.length() <= 0) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        // Check if the user account is disabled.
        try {
            if (0 != Integer.parseInt(theUserContext.getProp("disabled")))  {
                context.setUserText("User account is disabled.");

                return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_USER_DISABLED);
            }
        }
        catch (NumberFormatException exc) {
            // Do nothing -- the user is not disabled
        }

        // Authentication

        // Authenticate the user
        String authUserText;

        
        // Do the delay         
        try { 
            //log(context, "Starting Auth sleep "+ new Date()); 
            Thread.sleep(getDelayTimeMilli(parameter));
            //log(context, "ending Auth sleep "+ new Date()); 
        }
        catch (InterruptedException e) {}
        
        try {
            authUserText = theUserContext.authenticateUser(thePassword);
        } 
        catch (Throwable exc) {
            // insure subsequent code knows the authentication attempt failed^M
            authUserText = null;
        }

        if (null == authUserText){
            context.setErrorText("Unable to authenticate user " + theUserContext.getUserName());
            return  new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }
                
        // Ok we passed all that, so we are authenticated, note we did not call the password authenticate module, since we 
        // did not need to check the password, the fact this user existed in the user store, and had a Delayificat that matches
        // is enough.

        return new SmAuthenticationResult(SmAuthStatus.SMAUTH_ACCEPT, SmAuthenticationResult.REASON_NONE);
    }
}

// EOF AuthApiSample.java
